import { world, system } from "@minecraft/server";

// Track player positions and sound states
const playerPositions = new Map();
const lastSoundTime = new Map();
const monitoredPlayers = new Set(); // Track player-specific subscriptions
// Play custom sound
function playCustomSound(player, soundName, volume = 1) {
    const { x, y, z } = player.location;
    player.runCommandAsync(`playsound ${soundName} @a[r=10] ${x} ${y} ${z} ${volume} 1`);
}

// Stop custom sound
function stopCustomSound(player, soundName) {
    player.runCommandAsync(`stopsound @a ${soundName}`);
}

// Mob hurt sound map
const mobHurtSoundMap = {
    "minecraft:creeper": ["custom.echo_creeper_hurt1", "custom.echo_creeper_hurt2", "custom.echo_creeper_hurt3"],
    "minecraft:skeleton": ["custom.echo_skeleton_hurt1", "custom.echo_skeleton_hurt2", "custom.echo_skeleton_hurt3", "custom.echo_skeleton_hurt4"],
    "minecraft:zombie": ["custom.echo_zombie_hurt1", "custom.echo_zombie_hurt2"],
    "minecraft:spider": ["custom.echo_spider_hurt1", "custom.echo_spider_hurt2", "custom.echo_spider_hurt3"],
    "minecraft:bat": ["custom.echo_bat_hurt1", "custom.echo_bat_hurt2", "custom.echo_bat_hurt3", "custom.echo_bat_hurt4"],
    "minecraft:zombie_villager_v2": ["custom.echo_zombie_villager_hurt1", "custom.echo_zombie_villager_hurt2"]
};

// Mob death sound map
const mobDeathSoundMap = {
    "minecraft:creeper": "custom.echo_creeper_death",
    "minecraft:skeleton": "custom.echo_skeleton_death",
    "minecraft:zombie": "custom.echo_zombie_death",
    "minecraft:spider": "custom.echo_spider_death",
    "minecraft:bat": "custom.echo_bat_death",
    "minecraft:zombie_villager_v2": "custom.echo_zombie_villager_death"
};

// Map block types to specific sounds
const blockPlaceSoundMap = {
    "minecraft:amethyst_block": ["custom.echo_amethyst_place1", "custom.echo_amethyst_place2"],
    "minecraft:bell": ["custom.echo_bell_place1", "custom.echo_bell_place2"],
    "minecraft:deepslate": ["custom.echo_deepslate_place1", "custom.echo_deepslate_place2"],
    "minecraft:dirt": ["custom.echo_dirt_place1", "custom.echo_dirt_place2"],
    "minecraft:glass": ["custom.echo_glass_place1", "custom.echo_glass_place2"],
    "minecraft:granite": ["custom.echo_granite_place1", "custom.echo_granite_place2"],
    "minecraft:grass_block": ["custom.echo_grass_place1", "custom.echo_grass_place2"],
    "minecraft:gravel": ["custom.echo_gravel_place1", "custom.echo_gravel_place2"],
    "minecraft:torch": ["custom.echo_torch_place1", "custom.echo_torch_place2"],
    "minecraft:tuff": ["custom.echo_tuff_place1", "custom.echo_tuff_place2"],
    "minecraft:wood": ["custom.echo_wood_place1", "custom.echo_wood_place2"],
    "minecraft:white_wool": ["custom.echo_cloth_place1", "custom.echo_cloth_place2"],
    "minecraft:azalea": ["custom.echo_azalea_place1", "custom.echo_azalea_place2"],
    "minecraft:azalea_leaves": ["custom.echo_azalea_place1", "custom.echo_azalea_place2"],
    "minecraft:azalea_leaves_flowered": ["custom.echo_azalea_place1", "custom.echo_azalea_place2"],
    "minecraft:flowering_azalea": ["custom.echo_azalea_place1", "custom.echo_azalea_place2"],
    "minecraft:moss_block": ["custom.echo_grass_place1", "custom.echo_grass_place2"],
    "minecraft:moss_carpet": ["custom.echo_grass_place1", "custom.echo_grass_place2"],
    "minecraft:bed": ["custom.echo_wool_place1", "custom.echo_wool_place2"],
    "minecraft:crafting_table": ["custom.echo_wood_place1", "custom.echo_wood_place2"]
};

const defaultPlaceSounds = ["custom.echo_default_place1", "custom.echo_default_place2"];

const blockBreakSoundMap = {
    "minecraft:amethyst_block": ["custom.echo_amethyst_break1", "custom.echo_amethyst_break2"],
    "minecraft:bell": ["custom.echo_bell_break1", "custom.echo_bell_break2"],
    "minecraft:deepslate": ["custom.echo_deepslate_break1", "custom.echo_deepslate_break2"],
    "minecraft:dirt": ["custom.echo_dirt_break1", "custom.echo_dirt_break2"],
    "minecraft:glass": ["custom.echo_glass_break1", "custom.echo_glass_break2"],
    "minecraft:granite": ["custom.echo_granite_break1", "custom.echo_granite_break2"],
    "minecraft:grass_block": ["custom.echo_grass_break1", "custom.echo_grass_break2"],
    "minecraft:gravel": ["custom.echo_gravel_break1", "custom.echo_gravel_break2"],
    "minecraft:torch": ["custom.echo_torch_break1", "custom.echo_torch_break2"],
    "minecraft:tuff": ["custom.echo_tuff_break1", "custom.echo_tuff_break2"],
    "minecraft:wood": ["custom.echo_wood_break1", "custom.echo_wood_break2"],
    "minecraft:white_wool": ["custom.echo_cloth_break1", "custom.echo_cloth_break2"],
    "minecraft:azalea": ["custom.echo_azalea_break1", "custom.echo_azalea_break2"],
    "minecraft:azalea_leaves": ["custom.echo_azalea_break1", "custom.echo_azalea_break2"],
    "minecraft:azalea_leaves_flowered": ["custom.echo_azalea_break1", "custom.echo_azalea_break2"],
    "minecraft:flowering_azalea": ["custom.echo_azalea_break1", "custom.echo_azalea_break2"],
    "minecraft:moss_block": ["custom.echo_grass_break1", "custom.echo_grass_break2"],
    "minecraft:moss_carpet": ["custom.echo_grass_break1", "custom.echo_grass_break2"],
    "minecraft:bed": ["custom.echo_wool_break1", "custom.echo_wool_break2"],
    "minecraft:crafting_table": ["custom.echo_wood_break1", "custom.echo_wood_break2"]
};

const defaultBreakSounds = ["custom.echo_default_break1", "custom.echo_default_break2"];

// Handle mob hurt sounds
world.afterEvents.playerPlaceBlock.subscribe((event) => {
    const block = event.block;
    const player = event.player;
    if (block && block.location.y < 10) {
        const blockSound = blockPlaceSoundMap[block.typeId] || defaultPlaceSounds;
        const sound = blockSound[Math.floor(Math.random() * blockSound.length)];
        const blockName = block.typeId.substring(10);
        playCustomSound(player, sound);
        stopCustomSound(player, `block.${blockName}.place`);
    }
});

// Handle mob death sounds
world.beforeEvents.playerBreakBlock.subscribe((event) => {
    const block = event.block;
    const player = event.player;
    if (block && block.location.y < 10) {
        const blockSound = blockBreakSoundMap[block.typeId] || defaultBreakSounds;
        const sound = blockSound[Math.floor(Math.random() * blockSound.length)];
        const blockName = block.typeId.substring(10);
        playCustomSound(player, sound);
        stopCustomSound(player, `block.${blockName}.break`);
    }
});

// Handle mob hurt sounds
world.afterEvents.entityHurt.subscribe((event) => {
    const entity = event.hurtEntity;
    if (entity && entity.location.y < 10) {
        const entityName = entity.typeId.substring(10);
        const hurtSounds = mobHurtSoundMap[entity.typeId];
        const sound = hurtSounds[Math.floor(Math.random() * hurtSounds.length)];
        stopCustomSound(entity, `mob.${entityName}.hurt`);
        stopCustomSound(entity, `mob.${entityName}.say`);
        playCustomSound(entity, sound);
    }
});

// Handle mob death sounds
world.afterEvents.entityDie.subscribe((event) => {
    const entity = event.deadEntity;
    if (entity && entity.location.y < 10) {
        const entityName = entity.typeId.substring(10);
        const sound = mobDeathSoundMap[entity.typeId];
        stopCustomSound(entity, `mob.${entityName}.death`);
        playCustomSound(entity, sound);
    }
});

function trackPlayerMovement() {
    const WALK_DELAY_MS = 310;
    const RUN_DELAY_MS = 255;

    for (const player of world.getPlayers()) {
        const position = player.location;

        // Skip players above Y=0 or not on the ground
        if (position.y >= 10 || !player.isOnGround) continue;

        let lastPosition = playerPositions.get(player);
        if (!lastPosition) {
            lastPosition = { x: position.x, y: position.y, z: position.z };
            playerPositions.set(player, lastPosition);
        }

        const distanceMovedX = Math.abs(position.x - lastPosition.x);
        const distanceMovedZ = Math.abs(position.z - lastPosition.z);
        const totalDistanceMoved = Math.sqrt(distanceMovedX ** 2 + distanceMovedZ ** 2);

        const RUN_THRESHOLD = 1.5;
        const WALK_THRESHOLD = 0.5;

        const isSprinting = totalDistanceMoved >= RUN_THRESHOLD;
        const isWalking = totalDistanceMoved >= WALK_THRESHOLD && totalDistanceMoved < RUN_THRESHOLD;

        const currentTime = Date.now();
        const lastPlayedTime = lastSoundTime.get(player) || 0;
        const soundDelay = isSprinting ? RUN_DELAY_MS : WALK_DELAY_MS;

        if ((isSprinting || isWalking) && currentTime - lastPlayedTime >= soundDelay) {
            const soundIndex = Math.floor(Math.random() * 6) + 1;
            const soundName = `custom.echo_walk${soundIndex}`;
            playCustomSound(player, soundName);
            lastSoundTime.set(player, currentTime);
            playerPositions.set(player, { x: position.x, y: position.y, z: position.z });
        }
    }

    system.runTimeout(trackPlayerMovement, 1);
}

// Monitor all players for block actions and movement
function monitorPlayers() {
    for (const player of world.getPlayers()) {
        if (player.location.y < 0) {
            monitoredPlayers.add(player); // Add to the monitored set if below Y=0
        }
    }
    trackPlayerMovement();
}

// Handle new players joining
world.afterEvents.playerJoin.subscribe((event) => {
    const player = event.player;
    if (player.location.y < 0 && !monitoredPlayers.has(player)) {
        monitoredPlayers.add(player);
        monitorPlayers(); // Monitors players periodically
    }
});

// Initialize the script
system.run(monitorPlayers);

